﻿// ----------------------------------------------------------------------------------
//
// Copyright Microsoft Corporation
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// http://www.apache.org/licenses/LICENSE-2.0
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
// ----------------------------------------------------------------------------------

using System.Collections.Generic;
using System.Linq;
using System.Management.Automation;
using Microsoft.WindowsAzure.Commands.ServiceManagement.Model;
using Microsoft.WindowsAzure.Management.Compute;

namespace Microsoft.WindowsAzure.Commands.ServiceManagement.Extensions
{
    /// <summary>
    /// Get Microsoft Azure Service Antimalware Extension Configuration
    /// </summary>
    [Cmdlet(VerbsCommon.Get, "AzureServiceAntimalwareConfig"), OutputType(typeof(IEnumerable<AntimalwareExtensionContext>))]
    public class GetAzureServiceAntimalwareExtensionCommand : BaseAzureServiceAntimalwareExtensionCmdlet
    {
        [Parameter(Position = 0, ValueFromPipelineByPropertyName = true, Mandatory = true, HelpMessage = "Service Name")]
        [ValidateNotNullOrEmpty]
        public override string ServiceName { get; set; } 

        [Parameter(Position = 1, ValueFromPipelineByPropertyName = true, Mandatory = false, HelpMessage = "Deployment Slot: Production (default) or Staging")]
        [ValidateSet(DeploymentSlotType.Production, DeploymentSlotType.Staging, IgnoreCase = true)]
        public override string Slot { get; set; }

        protected override void ValidateParameters()
        {
            base.ValidateParameters();
            ValidateService();
            ValidateDeployment();
        }

        public void ExecuteCommand()
        {           
            ValidateParameters();

            ExecuteClientActionNewSM(
                null,
                CommandRuntime.ToString(),
                () => this.ComputeClient.HostedServices.ListExtensions(this.ServiceName),
                (s, r) =>
                {
                    var extensionRoleList = (from dr in Deployment.Roles
                                                select new ExtensionRole(dr.RoleName)).ToList().Union(new ExtensionRole[] { new ExtensionRole() });

                    return  from role in extensionRoleList
                            from extension in r.Extensions
                            where (ExtensionManager.CheckNameSpaceType(extension, AntimalwareExtensionProviderNamespace, AntimalwareExtensionName) || 
                                   ExtensionManager.CheckNameSpaceType(extension, AntimalwareDiagnosticsExtensionNamespace, AntimalwareDiagnosticsExtensionName))
                                && ExtensionManager.GetBuilder(Deployment.ExtensionConfiguration).Exist(role, extension.Id)
                            select new ExtensionContext
                            {
                                OperationId = s.Id,
                                OperationDescription = CommandRuntime.ToString(),
                                OperationStatus = s.Status.ToString(),
                                Extension = extension.Type,
                                Id = extension.Id,
                                Role = role,
                                ProviderNameSpace = extension.ProviderNamespace,
                                PublicConfiguration = extension.PublicConfiguration
                            };
                }); 
        }

        protected override void OnProcessRecord()
        {
            ExecuteCommand();
        }
    }
}
